package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;
 
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.*;
import javax.swing.event.*;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
 
/**
 * Panel for editing of Ports of a SmartDevice
 *
 * @author Andreas T. Meyer-Berg
 */
@SuppressWarnings("serial")
public class PortEditorPanel extends JPanel
                          implements ListSelectionListener, FocusListener, ActionListener, MouseListener {

	/**
	 * Panel, which contains the different Textfields and Selectors for editing a port
	 */
	private JPanel editPanel;
	/**
	 * List for the selection of a port
	 */
    private JList<Port> list;
    /**
     * Split pane for the List
     */
    private JSplitPane splitPane;
    
    /**
     * SmartDevice which is being edited
     */
    private SmartDevice toEdit;
    
    /**
     * Label for the protocol name
     */
    private JLabel lblProtocolName;
    /**
     * Textfield for the PortNumber
     */
    private JTextField tfPortNumber;
    /**
     * ComboBox for the status
     */
    private JComboBox<String> cmbStatus;
    /**
     * Textfield for the trigger Interval
     */
    private JTextField tfTriggerInterval;
    /**
     * Textfield for the responseTime
     */
    private JTextField tfResponseTime;
    /**
     * Textfield for the last trigger
     */
    private JTextField tfLastTrigger;
    
    /**
     * Creates a new PortEditorPanel, which allows editing of the Ports of toEdit
     * @param toEdit SmartDevice, which ports will be edited
     */
    public PortEditorPanel(SmartDevice toEdit) {
    	
    	this.toEdit = toEdit;
        //Create the list of ports
        Port[] ports = new Port[toEdit.getPorts().size()];
        for(int i = 0; i<toEdit.getPorts().size();i++)
        	ports[i]=toEdit.getPorts().get(i);
        
        list = new JList<Port>(ports);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setSelectedIndex(0);
        list.addListSelectionListener(this);
         
        
        JScrollPane listScrollPane = new JScrollPane(list);
        editPanel = new JPanel();
        editPanel.setFont(editPanel.getFont().deriveFont(Font.ITALIC));
        editPanel.setLayout(null);
        
        JLabel lblPortnumber = new JLabel("PortNumber:");
        lblPortnumber.setHorizontalAlignment(SwingConstants.RIGHT);
        lblPortnumber.setBounds(49, 8, 73, 16);
        editPanel.add(lblPortnumber);
         
        JScrollPane pictureScrollPane = new JScrollPane(editPanel);
        
        tfPortNumber = new JTextField();
        tfPortNumber.setBounds(134, 5, 132, 22);
        editPanel.add(tfPortNumber);
        tfPortNumber.setColumns(10);
        tfPortNumber.addFocusListener(this);
        tfPortNumber.addActionListener(this);
        tfPortNumber.addMouseListener(this);
        
        JLabel lblProtocol = new JLabel("Protocol:");
        lblProtocol.setHorizontalAlignment(SwingConstants.RIGHT);
        lblProtocol.setBounds(66, 37, 56, 16);
        editPanel.add(lblProtocol);
        
        JLabel lblStatus = new JLabel("Status:");
        lblStatus.setHorizontalAlignment(SwingConstants.RIGHT);
        lblStatus.setBounds(66, 66, 56, 16);
        editPanel.add(lblStatus);
        
        JButton btnEditConnection = new JButton("Edit");
        btnEditConnection.setBounds(193, 33, 73, 25);
        editPanel.add(btnEditConnection);
        
        lblProtocolName = new JLabel("Protocol");
        lblProtocolName.setBounds(134, 37, 56, 16);
        editPanel.add(lblProtocolName);
        
        cmbStatus = new JComboBox<String>();
        cmbStatus.addItem(Port.statusToString(Port.CLOSED));
        cmbStatus.addItem(Port.statusToString(Port.OPEN));
        cmbStatus.addItem(Port.statusToString(Port.SENDING));
        cmbStatus.addItem(Port.statusToString((short) 4));
        cmbStatus.setBounds(134, 63, 132, 22);
        editPanel.add(cmbStatus);
        cmbStatus.addFocusListener(this);
        cmbStatus.addActionListener(this);
        cmbStatus.addMouseListener(this);
        JLabel lblTriggerInterval = new JLabel("Trigger Interval:");
        lblTriggerInterval.setHorizontalAlignment(SwingConstants.RIGHT);
        lblTriggerInterval.setBounds(0, 95, 122, 16);
        editPanel.add(lblTriggerInterval);
        
        tfTriggerInterval = new JTextField();
        tfTriggerInterval.setBounds(134, 92, 132, 22);
        editPanel.add(tfTriggerInterval);
        tfTriggerInterval.setColumns(10);
        tfTriggerInterval.addFocusListener(this);
        tfTriggerInterval.addActionListener(this);
        tfTriggerInterval.addMouseListener(this);
        
        JLabel lblResponseTime = new JLabel("Response Time:");
        lblResponseTime.setHorizontalAlignment(SwingConstants.RIGHT);
        lblResponseTime.setBounds(10, 124, 112, 16);
        editPanel.add(lblResponseTime);
        
        tfResponseTime = new JTextField();
        tfResponseTime.setBounds(134, 121, 132, 22);
        editPanel.add(tfResponseTime);
        tfResponseTime.setColumns(10);
        tfResponseTime.addFocusListener(this);
        tfResponseTime.addActionListener(this);
        tfResponseTime.addMouseListener(this);
        
        JLabel lblLasttrigger = new JLabel("LastTrigger:");
        lblLasttrigger.setHorizontalAlignment(SwingConstants.RIGHT);
        lblLasttrigger.setBounds(20, 153, 102, 16);
        editPanel.add(lblLasttrigger);
        
        tfLastTrigger = new JTextField();
        tfLastTrigger.setBounds(134, 150, 132, 22);
        editPanel.add(tfLastTrigger);
        tfLastTrigger.setColumns(10);
        tfLastTrigger.addFocusListener(this);
        tfLastTrigger.addActionListener(this);
        tfLastTrigger.addMouseListener(this);
        
        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                   listScrollPane, pictureScrollPane);
        splitPane.setDividerLocation(60);
 
        /**
         * minimum sizes of the panels
         */
        Dimension minimumSize = new Dimension(60, 50);
        listScrollPane.setMinimumSize(minimumSize);
        listScrollPane.setMaximumSize(minimumSize);
        pictureScrollPane.setMinimumSize(minimumSize);
 
        // Set the preferred size
        splitPane.setPreferredSize(new Dimension(400, 200));
        if(!toEdit.getPorts().isEmpty())
        	updateLabel(toEdit.getPorts().get(list.getSelectedIndex()));
    }
     
    //Listens to the list
    @SuppressWarnings("unchecked")
	public void valueChanged(ListSelectionEvent e) {
    	if(!(e.getSource() instanceof JList<?>)||!(((JList<?>)e.getSource()).getSelectedValue() instanceof Port))return;
        JList<Port> list = (JList<Port>)e.getSource();
        updateLabel(toEdit.getPorts().get(list.getSelectedIndex()));
    }
     
    /**
     * Updates the labels to the given Port
     * @param port Port which should be showed in this panel
     */
    protected void updateLabel (Port port) {
    	if(port == null)return;
    	tfPortNumber.setText(""+port.getPortNumber());
    	tfPortNumber.setBackground(Color.WHITE);
    	tfLastTrigger.setText(""+port.getLastTrigger());
    	tfLastTrigger.setBackground(Color.WHITE);
    	tfResponseTime.setText(""+port.getResponseTime());
    	tfResponseTime.setBackground(Color.WHITE);
    	tfTriggerInterval.setText(""+port.getTriggerInterval());
    	tfTriggerInterval.setBackground(Color.WHITE);
    	cmbStatus.setSelectedIndex(port.getStatus());
    	if(port.getConnection() == null || port.getConnection().getProtocol()==null)
    		lblProtocolName.setText("null");
    	else
    		lblProtocolName.setText(port.getConnection().getProtocol().getName());
    }
    
    /**
     * Returns the SplitPane, which is the Main component of this Panel
     * @return panel, which represents this object
     */
    public JSplitPane getSplitPane() {
        return splitPane;
    }
 
    /**
     * Test the Panel
     */
    private static void testGUI() {
        JFrame frame = new JFrame("SplitPaneDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        SmartDevice test = new SmartDevice("TestDevice");
        test.addPort(new Port(test, (short) 1));
        test.addPort(new Port(test, (short) 2));
        PortEditorPanel splitPaneDemo = new PortEditorPanel(test);
        frame.getContentPane().add(splitPaneDemo.getSplitPane());
 
        frame.pack();
        frame.setVisible(true);
    }
 
    public static void main(String[] args) {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                testGUI();
            }
        });
    }

    /**
     * Checks values of the textfields have changed, and updates the Protocol accordingly.
     * Invalid values are highlighted RED
     */
	private void checkValueChange() {
		if(list.getSelectedIndex()==-1||list.getSelectedIndex()>=toEdit.getPorts().size())return;
		/**
		 * Port, which is being edited at the moment 
		 */
		Port toChange = toEdit.getPorts().get(list.getSelectedIndex());
		
		//Edit Port Number
		if(tfPortNumber.getText()!=""+toChange.getPortNumber()){
			try {
				toChange.setPortNumber(Short.parseShort(tfPortNumber.getText()));
				tfPortNumber.setBackground(Color.WHITE);
			} catch (Exception e) {
				tfPortNumber.setBackground(Color.RED);
			}
		}else{
			tfPortNumber.setBackground(Color.WHITE);
		}
		
		//Edit Status
		if(cmbStatus.getSelectedIndex()!=toChange.getStatus())
			toChange.setStatus((short) cmbStatus.getSelectedIndex());
		
		//Edit trigger Interval
		if(tfTriggerInterval.getText()!=""+toChange.getTriggerInterval()){
			try {
				toChange.setTriggerInterval(Long.parseLong(tfTriggerInterval.getText()));
				tfTriggerInterval.setBackground(Color.WHITE);
			} catch (Exception e) {
				tfTriggerInterval.setBackground(Color.RED);
			}
		}else{
			tfTriggerInterval.setBackground(Color.WHITE);
		}
		
		//Edit Response time
		if(tfResponseTime.getText()!=""+toChange.getResponseTime()){
			try {
				toChange.setResponseTime(Short.parseShort(tfResponseTime.getText()));
				tfResponseTime.setBackground(Color.WHITE);
			} catch (Exception e) {
				tfResponseTime.setBackground(Color.RED);
			}
		}else{
			tfResponseTime.setBackground(Color.WHITE);
		}
		
		//Edit last Trigger
		if(tfLastTrigger.getText()!=""+toChange.getTriggerInterval()){
			try {
				toChange.setLastTrigger(Long.parseLong(tfLastTrigger.getText()));
				tfLastTrigger.setBackground(Color.WHITE);
			} catch (Exception e) {
				tfLastTrigger.setBackground(Color.RED);
			}
		}else{
			tfLastTrigger.setBackground(Color.WHITE);
		}
		
		
	}

	@Override
	public void focusGained(FocusEvent e) {
		checkValueChange();
	}

	@Override
	public void focusLost(FocusEvent e) {
		checkValueChange();
	}

	@Override
	public void mouseClicked(MouseEvent e) {	
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
		checkValueChange();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		checkValueChange();
	}
}